@@ -122,6 +122,53 @@
 #define TODD_COXETER_REPORT_OK()
 #endif
 
+// Helper functions
+namespace {
+  using class_index_type = libsemigroups::CongruenceInterface::class_index_type;
+  using word_type        = libsemigroups::word_type;
+
+  void sort_generating_pairs(std::vector<class_index_type>& perm,
+                             std::vector<word_type>&        vec) {
+    // Apply the permutation (adapted from
+    // stl.hpp:apply_permutation)
+    size_t const n = perm.size();
+    for (size_t i = 0; i < n; ++i) {
+      size_t current = i;
+      while (i != perm[current]) {
+        size_t next = perm[current];
+        std::swap(vec[2 * current], vec[2 * next]);
+        std::swap(vec[2 * current + 1], vec[2 * next + 1]);
+        perm[current] = current;
+        current       = next;
+      }
+      perm[current] = current;
+    }
+  }
+
+  void sort_generating_pairs(
+      std::function<bool(word_type const&, word_type const&)> func,
+      std::vector<class_index_type>&                          perm,
+      std::vector<word_type>&                                 vec) {
+    // Sort each relation so that the lhs is greater than the rhs according
+    // to func.
+    for (auto it = vec.begin(); it < vec.end(); it += 2) {
+      if (func(*it, *(it + 1))) {
+        std::swap(*it, *(it + 1));
+      }
+    }
+
+    // Create a permutation of the even indexed entries in vec
+    perm.resize(vec.size() / 2);
+    std::iota(perm.begin(), perm.end(), 0);
+    std::sort(perm.begin(),
+              perm.end(),
+              [&func, &vec](class_index_type x, class_index_type y) -> bool {
+                return func(vec[2 * x], vec[2 * y]);
+              });
+    sort_generating_pairs(perm, vec);
+  }
+}  // namespace
+
 namespace libsemigroups {
   namespace congruence {
     ////////////////////////////////////////////////////////////////////////
@@ -436,7 +483,7 @@ namespace libsemigroups {
     ////////////////////////////////////////////////////////////////////////
 
     // Init
-    void ToddCoxeter::prefill(Table const& table) {
+    void ToddCoxeter::prefill(table_type const& table) {
       prefill_and_validate(table, true);
       init_preimages_from_table();
     }
@@ -513,6 +560,35 @@ namespace libsemigroups {
       return *this;
     }
 
+    ToddCoxeter& ToddCoxeter::sort_generating_pairs(
+        std::function<bool(word_type const&, word_type const&)> func) {
+      if (started()) {
+        LIBSEMIGROUPS_EXCEPTION(
+            "Cannot sort relations, the coset enumeration has started!")
+      }
+      init();
+      std::vector<class_index_type> perm;
+      ::sort_generating_pairs(func, perm, _relations);
+      ::sort_generating_pairs(func, perm, _extra);
+      return *this;
+    }
+
+    ToddCoxeter& ToddCoxeter::random_shuffle_generating_pairs() {
+      if (started()) {
+        LIBSEMIGROUPS_EXCEPTION(
+            "Cannot shuffle relations, the coset enumeration has started!")
+      }
+      init();
+      std::vector<class_index_type> perm(0, _relations.size());
+      std::iota(perm.begin(), perm.end(), 0);
+      std::random_device rd;
+      std::mt19937       g(rd());
+      std::shuffle(perm.begin(), perm.end(), g);
+      ::sort_generating_pairs(perm, _relations);
+      ::sort_generating_pairs(perm, _extra);
+      return *this;
+    }
+
     ////////////////////////////////////////////////////////////////////////
     // ToddCoxeter - member functions (container-like) - public
     ////////////////////////////////////////////////////////////////////////
@@ -651,20 +727,27 @@ namespace libsemigroups {
     std::shared_ptr<FroidurePinBase> ToddCoxeter::quotient_impl() {
       using detail::TCE;
       run();
-      LIBSEMIGROUPS_ASSERT(finished());
-      if (!is_standardized()) {
-        standardize(order::shortlex);
+      standardize(order::shortlex);
+      shrink_to_fit();
+      // Ensure class indices and letters are equal!
+      auto   table = std::make_shared<table_type>(_table);
+      size_t n     = nr_generators();
+      for (letter_type a = 0; a < n;) {
+        if (table->get(0, a) != a + 1) {
+          table->erase_column(a);
+          n--;
+        } else {
+          ++a;
+        }
       }
-
-      // TODO(later) replace with 0-parameter constructor when available
-      std::vector<TCE> gens;
-      TCE              x(this);
+      auto ptr = std::make_shared<
+          FroidurePin<TCE, FroidurePinTraits<TCE, table_type>>>(table);
       for (size_t i = 0; i < nr_generators(); ++i) {
-        // We use _table.get(0, i) instead of just i, because there might be
+        // We use table.get(0, i) instead of just i, because there might be
         // more generators than cosets.
-        gens.emplace_back(x, _table.get(0, i));
+        ptr->add_generator(TCE(_table.get(0, i)));
       }
-      return std::make_shared<FroidurePin<TCE>>(gens);
+      return ptr;
     }
 
     void ToddCoxeter::run_impl() {
@@ -751,18 +834,18 @@ namespace libsemigroups {
 
     void ToddCoxeter::set_nr_generators_impl(size_t n) {
       // TODO(later) add columns to make it up to n?
-      _preim_init = Table(n, 1, UNDEFINED);
-      _preim_next = Table(n, 1, UNDEFINED);
-      _table      = Table(n, 1, UNDEFINED);
+      _preim_init = table_type(n, 1, UNDEFINED);
+      _preim_next = table_type(n, 1, UNDEFINED);
+      _table      = table_type(n, 1, UNDEFINED);
     }
 
     ////////////////////////////////////////////////////////////////////////
     // ToddCoxeter - member functions (validation) - private
     ////////////////////////////////////////////////////////////////////////
 
-    void ToddCoxeter::validate_table(Table const& table,
-                                     size_t const first,
-                                     size_t const last) const {
+    void ToddCoxeter::validate_table(table_type const& table,
+                                     size_t const      first,
+                                     size_t const      last) const {
       REPORT_DEBUG_DEFAULT("validating coset table...\n");
       if (nr_generators() == UNDEFINED) {
         LIBSEMIGROUPS_EXCEPTION("no generators have been defined");
@@ -837,10 +920,12 @@ namespace libsemigroups {
             REPORT_DEBUG_DEFAULT("using presentation...\n");
             LIBSEMIGROUPS_ASSERT(_settings->froidure_pin
                                  == policy::froidure_pin::use_relations);
-            auto sucker = [this](word_type const& w) -> void {
-              reverse_if_necessary_and_push_back(w, _relations);
-            };
-            relations(*parent_froidure_pin(), sucker);
+            auto fp = parent_froidure_pin();
+            fp->run();
+            for (auto it = fp->cbegin_rules(); it != fp->cend_rules(); ++it) {
+              reverse_if_necessary_and_push_back(it->first, _relations);
+              reverse_if_necessary_and_push_back(it->second, _relations);
+            }
 #ifdef LIBSEMIGROUPS_DEBUG
             // This is a check of program logic, since we use parent() to
             // obtain the relations, so we only validate in debug mode.
@@ -917,7 +1002,8 @@ namespace libsemigroups {
       init_preimages_from_table();
     }
 
-    void ToddCoxeter::prefill_and_validate(Table const& table, bool validate) {
+    void ToddCoxeter::prefill_and_validate(table_type const& table,
+                                           bool              validate) {
       if (_settings->strategy == policy::strategy::felsch) {
         LIBSEMIGROUPS_EXCEPTION(
             "it is not possible to prefill when using the Felsch strategy");
@@ -1405,7 +1491,7 @@ namespace libsemigroups {
         }
       }
       apply_permutation(p, q);
-      REPORT("%d\n", tmr.string()).prefix().flush_right().flush();
+      REPORT("%s\n", tmr.string()).prefix().flush_right().flush();
 #ifdef LIBSEMIGROUPS_DEBUG
       debug_validate_forwd_bckwd();
       debug_validate_table();
